"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const ts_simple_ast_1 = require("ts-simple-ast");
const ts_simple_ast_extra_1 = require("ts-simple-ast-extra");
function getTargetNode(sourceFile, position) {
    const target = typescript_ast_util_1.findChildContainingRangeLight(sourceFile, typescript_ast_util_1.positionOrRangeToRange(position));
    const predicate = (p) => ts.isCallSignatureDeclaration(p) || ts.isFunctionLike(p) || ts.isCallOrNewExpression(p) || ts.isMethodSignature(p) || ts.isConstructSignatureDeclaration(p);
    return typescript_ast_util_1.findAscendant(target, predicate, true);
}
exports.getTargetNode = getTargetNode;
function getTargetInfo(sourceFile, position, predicate = n => true) {
    let targetNode = getTargetNode(sourceFile, position);
    if (!targetNode) {
        return;
    }
    let parameterCount = 0, argumentCount = 0;
    let name;
    if (ts.isFunctionLike(targetNode)) {
        if (!predicate(targetNode)) {
            return;
        }
        name = targetNode.name ? targetNode.name.getText() : (targetNode.parent && targetNode.parent.name && targetNode.parent.name) ? targetNode.parent.name.getText() : undefined;
        if (!name) {
            return;
        }
        parameterCount = targetNode.parameters.length;
    }
    else if (ts.isCallExpression(targetNode)) {
        if (!predicate(targetNode)) {
            return;
        }
        if (!ts.isIdentifier(targetNode.expression)) {
            return;
        }
        name = targetNode.expression.getText();
        argumentCount = targetNode.arguments.length;
    }
    //TODO: isCallNew and others that comply with : ts.isCallSignatureDeclaration(p)||ts.isFunctionLike(p)||ts.isCallOrNewExpression(p
    else {
        // this.options.log('snippet undefined because not isCallExpression and not isFunctionLike' + targetNode.getText() + ' - ' + getKindName(targetNode))
        return;
    }
    return { name, argumentCount, parameterCount, targetNode };
}
exports.getTargetInfo = getTargetInfo;
/**
 * collect al references of given node and returns those nodes that need to be refactored
 */
function getAllCallsExpressions(targetDeclaration, log) {
    const calls = [];
    for (const referencedSymbol of targetDeclaration.findReferences()) {
        for (const reference of referencedSymbol.getReferences()) {
            const parent = reference.getNode().getParent();
            const extras = [parent, parent.getParent && parent.getParent(), targetDeclaration];
            const collection = (extras.concat(ts_simple_ast_extra_1.getChildrenForEachChild(parent)))
                .filter((value, pos, arr) => arr.indexOf(value) === pos);
            const found = collection
                .find(p => (!ts_simple_ast_extra_1.getName(p) || ts_simple_ast_extra_1.getName(targetDeclaration) === ts_simple_ast_extra_1.getName(p)) &&
                (ts_simple_ast_1.TypeGuards.isCallExpression(p) || ts_simple_ast_1.TypeGuards.isSignaturedDeclaration(p)));
            if (found) {
                calls.push(found);
            }
            else {
                log('getAllCallsExpressions ignoring reference parent: ' + parent.getKindName() +
                    ' ref: ' + reference.getNode().getKindName() +
                    ', chh ' + parent.getChildren().map(c => c.getKindName()).join(', ') +
                    ' ancestors: ' + parent.getAncestors().map(c => c.getKindName()).join(', '));
            }
        }
    }
    return calls.filter((value, pos, arr) => arr.indexOf(value) === pos);
}
exports.getAllCallsExpressions = getAllCallsExpressions;
function applyToSignature(node, fn, log = console.log) {
    if (ts_simple_ast_1.TypeGuards.isReferenceFindableNode(node)) {
        getAllCallsExpressions(node, log).forEach(call => {
            if (ts_simple_ast_1.TypeGuards.isCallExpression(call)) {
                fn(call.getArguments(), call, null);
            }
            else {
                fn(call.getParameters(), call, null);
            }
        });
    }
    else { // if(TypeGuards.isConstructorDeclaration(node)) {
        log('!TypeGuards.isReferenceFindableNode(node) ' + node.getText());
        //TODO: find interface constructor implememted by this one and rename that one and then run this function calling it with node.getAncestors().find(TypeGuards.isReferenceFindableNode) to rename new A calls
        // return []
    }
}
exports.applyToSignature = applyToSignature;
//# sourceMappingURL=util.js.map